#include "io.h"
#include "pwm.h"
#include <wirish_time.h>
#include <stm32/l1/gpio.h>
#include <stm32/l1/tim.h>
#include <stm32/l1/adc.h>
#include <stm32/l1/dac.h>

void pinMode(uint8 pin, WiringPinMode mode) {
    gpio_otype_t outputMode;
    gpio_pupd_t pupdMode = GPIO_NOPUPD;
    bool output = false;

    switch(mode) {
    case OUTPUT:
        output = true;
        outputMode = GPIO_PUSHPULL;
        break;
    case OUTPUT_OPEN_DRAIN:
        output = true;
        outputMode = GPIO_OPENDRAIN;
        break;
    case INPUT:
    case INPUT_FLOATING:
    case INPUT_ANALOG:
        pupdMode = GPIO_FLOAT;
        break;
    case INPUT_PULLUP:
        pupdMode = GPIO_PULLUP;
        break;
    case INPUT_PULLDOWN:
        pupdMode = GPIO_PULLDOWN;
        break;
    default:
        return;
    }

    if (output) {
        if (pin == BOARD_DAC_PIN) {
            gpio_config_analog(PIN_MAP[pin].gpio_pin);
            /* Enable DAC channel1. */
            dac_enable(DAC_CH1);

            /* Wait tWAKEUP. */
            delayMicroseconds(DAC_T_WAKEUP);
        } else if (PIN_MAP[pin].timer_device != TIMx) { //port kann PWM
            gpio_config_altfn(PIN_MAP[pin].altfn, GPIO_PUSHPULL, GPIO_2MHZ, GPIO_NOPUPD,
                              PIN_MAP[pin].gpio_pin);

        } else {
            gpio_config_output(outputMode, GPIO_2MHZ, GPIO_NOPUPD,
                               PIN_MAP[pin].gpio_pin);
            gpio_clear(PIN_MAP[pin].gpio_pin);
        }
    } else { //input
        if (PIN_MAP[pin].adc_channel != ADCx) //port kann ADC
            gpio_config_analog(PIN_MAP[pin].gpio_pin);
        else
            gpio_config_input(pupdMode, PIN_MAP[pin].gpio_pin);
    }

}


void togglePin(uint8 pin) {
    if (PIN_MAP[pin].timer_device != TIMx)
        analogWrite(pin,
                    tim_get_capture_compare_value(PIN_MAP[pin].timer_channel) ? 0 : 255);
    else
        gpio_toggle(PIN_MAP[pin].gpio_pin);
}

void digitalWrite(uint8 pin, uint8 value)
{
    if (PIN_MAP[pin].timer_device != TIMx) //port kann PWM
        analogWrite(pin, value ? 255 : 0);
    else if (value)
        gpio_set(PIN_MAP[pin].gpio_pin);
    else
        gpio_clear(PIN_MAP[pin].gpio_pin);
}

uint32 digitalRead(uint8 pin)
{
    return gpio_get(PIN_MAP[pin].gpio_pin);
}

void analogWrite(uint8 pin, int duty_cycle8)
{
    if (pin == BOARD_DAC_PIN)
        dac_set_data(DAC_8R, DAC_CH1, duty_cycle8);
    else if (PIN_MAP[pin].timer_device != TIMx)
        tim_set_capture_compare_value(PIN_MAP[pin].timer_channel, duty_cycle8);
    else
        digitalWrite(pin, duty_cycle8 > 127 ? 1 : 0);
}

uint16 analogRead(uint8 pin)
{
    uint16 ret;

    /* Set channel number */
    adc_set_regular_channel(PIN_MAP[pin].adc_channel);

    /* ADC on */
	adc_enable();

    /* 10bit resolution */
    adc_set_resolution(10);

    /* ADC power-up time (tSTAB) */
    delayMicroseconds(ADC_T_STAB);

    /* Start conversion. */
    adc_start_regular_conversion();

    /* Wait for the end of conversion. */
    while (!adc_get_interrupt_status(ADC_REGULAR_END))
        ;

    /* Get data and clear interrupt. */
    ret = adc_get_regular_data();

    /* ADC off */
	adc_disable();

    return ret;
}
